#!/usr/bin/env bash

# This bash-script reads the 2m temperature from ERA5 short-range forecasts files, crops the data to the largest target domain (see Dataset ID 1-3) and then performs a merging step with the ground truth and the persistence forecast data provided by from the postprocessing of a trained model (see prostprocess-step of workflow.)

# Parse script argument
yr=$1

# some directory parameters
indir=<path_to_era5_short_range_forecasts>/${yr}       # directory where the ERA5-short range forecasts are located
indir_ref=<path_to_postprocessed_model>                # directory where the results of a trained model are located (to get persistence forecast and ground truth data)
outdir=<path_to_output>                                # output directory

if [[ ! -d "${outdir}" ]]; then
    mkdir ${outdir}
fi

# additional parameters
hh_s=6                        # start of lead time range in hours
hh_e=12                       # end of lead time range in hours

declare -a hh_list=(18)       # initialization hour of ERA5 forecasts to evaluate (either 6 or 18)

for hh in ${hh_list[@]}; do
    hh0=$(printf "%02d" ${hh})

    # slice and retrieve 2m temperature from forecast-files
    for mm in {01..12};
        do sf_files=(`ls ${indir}/${mm}/fc_${hh0}/*_{6..12}_sf_fc.grb`);
            for sf_file in ${sf_files[*]};
                do newfile=`basename "${sf_file}"`
                newfile="${outdir}/${newfile/.grb/.nc}"
                echo "Processing file '${newfile}'"
                cdo --eccodes -f nc copy -selname,2t -sellonlatbox,0.,27.3,38.4,54.9 ${sf_file} ${newfile}
            done
        done

    # merge all ERA5 forecasts in monthly netCDF-files
    date1=`date -d "${yr}0101" '+%Y%m%d'`
    date2=`date -d "${yr}0101 ${hh0}" '+%Y-%m-%d %H:%M:00'`
    date_end=`date -d "{yr}1231 ${hh0}" '+%Y%m%d'`

    while [[ "$date1" -le "$date_end" ]]; do
	      outfile=${outdir}/${date1}${hh0}_allfc.nc
        echo "Merging forecats for run at ${date2}..."
        cdo mergetime ${outdir}/${date1}_${hh}00_*.nc ${outfile}
        echo "Manipulate attributes and dimensions..."
        ncrename -O -v 2t,2t_era5_fcst ${outfile} ${outfile}
        ncap2 -O -s init_time=0 ${outfile} ${outfile}
        ncatted -O -a calendar,init_time,a,c,"proleptic_gregorian" -a units,init_time,a,c,"hours since ${date2}" ${outfile} ${outfile}
        ncrename -O -d time,fcst_hour -v time,fcst_hour ${outfile} ${outfile}
        ncap2 -O -s 'fcst_hour=int(fcst_hour)' ${outfile} ${outfile}
	
	# add ground truth data and persistence forecast from trained model
	patt=${indir_ref}/vfp_date_${date1}${hh0}_sample_ind*.nc
	if ls $patt 1>/dev/null 2<&1; then
	   file_src=`ls $patt`
	   ncks -d fcst_hour,5,11 -v 2t_ref,2t_persistence_fcst -A ${file_src} ${outfile}
	else 
	   echo "Could not find reference data for ${date2}..."
	   mv ${outfile} "${oufile/.nc/_ref_miss.nc}"
	fi

        date1="$(date -u --date="$date1 tomorrow" '+%Y%m%d')"
        date2="$(date -u --date="$date2 tomorrow" '+%Y-%m-%d %H:%M:00')"
   done
done
    
